jls.loader.provide('jls.util.Resource');

jls.loader.require('jls.util.Formatter');
jls.loader.require('jls.util.XML');

jls.util.Resource = jls.lang.Class.create( /** @lends jls.util.Resource.prototype */
{
    /**
     * Creates an Resource.
     * 
     * @constructs
     * @class This class provides resource labels.
     */
    initialize : function() {
        this._bundle = {};
    },
    set : function(key, value) {
        if (typeof value == 'undefined') {
            delete this._bundle[key];
        } else {
            this._bundle[key] = value;
        }
        return this;
    },
    /**
     * Returns a label.
     *
     * @param {String} key The label key.
     * @returns {String} The label string.
     */
    get : function(key) {
        if (typeof key != 'string') {
            throw new jls.lang.Exception('Missing key argument');
        }
        if (key in this._bundle) {
            return this._bundle[key];
        }
        return key;
    },
    /**
     * Returns a formatted label.
     *
     * @param {String} key The formatted label key.
     * @returns {String} The formatted label string.
     */
    getf : function(key) {
        var args = Array.from(arguments);
        var key = args.shift();
        var format = this.get(key);
        if (arguments.length < 2) {
            return format;
        }
        args.unshift(format);
        return jls.util.Formatter.format.apply(jls.util.Formatter, args);
    }
});

Object.extend(jls.util.Resource, /** @lends jls.util.Resource */
{
    /**
     * The default resource.
     * @type jls.util.Resource
     */
    DEFAULT: new jls.util.Resource(),
    loadFromXML : function(xml) {
        if (jls.logger.isTraceEnabled()) {
            jls.logger.trace('loadFromXML("' + xml.toString() + '")');
        }
        var res = new jls.util.Resource();
        var entries = xml.getChildren();
        for (var i = 0; i < entries.length; i++) {
            var entry = entries[i];
            var key = entry.getAttribute('key');
            if ((entry.getName() == 'entry') && (key != null)) {
                res.set(key, entry.getText());
            }
        }
        return res;
    },
    getPath : function(path, language) {
        if (typeof language == 'undefined') {
            language = _native.core.properties['user.language'];
        }
        return path + '.' + language + '.xml';
    },
    _getResourceAsXML : function(path) {
        if ('_getResource' in _native.core) {
            var xml = _native.core._getResource(path, true);
            return jls.util.XML.createFromDOM(xml);
        } else {
            var xml = new XML(_native.core.getResourceAsString(path));
            return jls.util.XML.createFromE4X(xml);
        }
    },
    /**
     * Loads a resource from an XML path.
     *
     * @param {String} path The path of the XML resource to load.
     * @param {String} [language] The language to use.
     * @returns {jls.util.Resource} The resource.
     */
    load : function(path, language) {
        /*
         * TODO also try to load without language?
         */
        var xml = jls.util.Resource._getResourceAsXML(jls.util.Resource.getPath(path, language));
        return jls.util.Resource.loadFromXML(xml);
    }
});
