jls.loader.provide('jls.util.XMLHttpRequest');

jls.util.XMLHttpRequest = jls.lang.Class.create({
    /**
     * Creates an XMLHttpRequest.
     * 
     * @param {String} url The URL.
     * @constructs
     * @class This class allows to perform HTTP requests.
     */
    initialize : function(url) {
        this._url = url;
        this._method = 'get';
        this._contentType = 'application/x-www-form-urlencoded'; // 'text/plain';
        this._user = null;
        this._password = null;
        this._async = true;
    },
    /**
     * Sets the authentication credentials.
     * 
     * @param {String} user The authentication user.
     * @param {String} password The authentication password.
     * @returns {jls.util.XMLHttpRequest} this XMLHttpRequest.
     */
    setAuthentication : function(user, password) {
        this._user = user;
        this._password = password;
        return this;
    },
    /**
     * Sets the HTTP content type.
     * 
     * @param {String} contentType The HTTP content type.
     * @returns {jls.util.XMLHttpRequest} this XMLHttpRequest.
     */
    setContentType : function(contentType) {
        this._contentType = contentType;
        return this;
    },
    /**
     * Sets the HTTP method.
     * 
     * @param {String} method The HTTP method.
     * @returns {jls.util.XMLHttpRequest} this XMLHttpRequest.
     */
    setMethod : function(method) {
        this._method = method;
        return this;
    },
    /**
     * Adjusts this request's asynchronous mode.
     * 
     * @param {Boolean} asynchronous Whether the request is placed in asynchronous mode.
     * @returns {jls.util.XMLHttpRequest} this XMLHttpRequest.
     */
    configureAsynchronous : function(asynchronous) {
        this._async = asynchronous;
        return this;
    },
    /**
     * Sends the HTTP request.
     * 
     * @param {String} [data] The HTTP body to send.
     * @param {Function} [onComplete] The function to call when the request completes.
     * @param {Function} [onException] The function to call when an exception is thrown.
     */
    send : function(data, onComplete, onException) {
        new _native.boot.httpRequest(this._url, {
            method : this._method,
            contentType : this._contentType,
    	    user : this._user,
    	    password : this._password,
            postBody : data || null,
            asynchronous : this._async,
            onComplete : onComplete || jls.lang.Class.emptyFunction,
            onException : onException || jls.util.XMLHttpRequest.DEFAULT_ON_EXCEPTION,
            context : this
        }).send();
    }
});

Object.extend(jls.util.XMLHttpRequest, /** @lends jls.util.XMLHttpRequest */
{
    DEFAULT_ON_EXCEPTION: function(e) {
        //jls.logger.warn('Uncaught exception during exchange processing "' + e + '"');
        setTimeout(function() {
            throw new jls.lang.Exception('Uncaught exception during request processing', e);
        }, 10);
    }
});
