jls.loader.provide('spyl.pb.Media');

spyl.pb.Media = jls.lang.Class.create({
    initialize : function(id, userId, date, pubDate, description, filename, commentCount) {
        this._id = id;
        this._userId = userId;
        this._date = date;
        this._pubDate = pubDate;
        this._description = description;
        this._filename = filename;
        this._commentCount = commentCount;
        this._comments = [];
    },
    getTitle : function() {
        var launcher = spyl.pb.Launcher.getInstance();
        var title = launcher.formatDate(this._date);
        if (this._description) {
            title += ', ' + this._description;
        }
        //title += ', ' + this._filename;
        title += ', ' + launcher.users[this._userId].login;
        if (this._commentCount > 0) {
            title += ', ' + this._commentCount + ' ';
            if (this._commentCount > 1) {
                title += launcher.commentsLabel;
            } else {
                title += launcher.commentLabel;
            }
        }
        return title;
    },
    getId : function() {
        return this._id;
    },
    getDate : function() {
        return this._date;
    },
    addComment : function(comment) {
        this._comments.push(comment);
    },
    newComment : function(userId, content) {
        this._comments.push(new spyl.pb.Comment(-1, userId, this._id, content, new Date()));
        this._commentCount++;
    },
    getCommentCount : function() {
        return this._commentCount;
    },
    getComments : function() {
        return this._comments;
    },
    getBasePathname: function() {
        return 'private/' + this._userId.toPaddedString(4) + '/' + this._date.getFullYear() + '/';
    },
    getPathname: function(ext) {
        return this.getBasePathname() + this._id + ext;
    },
    getMiniaturePathname: function() {
        return this.getBasePathname() + 'mini/' + this._id + '.jpg';
    },
    createMiniature : function(params, parent) {
        jls.logger.trace('createMiniature()');
        return new spyl.pb.PreviewElement(params, parent);
    }
});

Object.extend(spyl.pb.Media,
{
    compareMedias : function(a, b) {
        if (a._date < b._date)
            return -1;
        if (a._date > b._date)
            return 1;
        return 0;
    },
    compareMediasByPubDate : function(a, b) {
        if (a._pubDate < b._pubDate)
            return -1;
        if (a._pubDate > b._pubDate)
            return 1;
        return 0;
    },
    createFromXML : function(node) {
        var type = parseInt(node.getAttribute('type'), 10);
        var id = parseInt(node.getAttribute('id'), 10);
        var type = parseInt(node.getAttribute('type'), 10);
        var userId = parseInt(node.getAttribute('userId'), 10);
        var filename = node.getAttribute('filename');
        var description = node.getAttribute('description');
        var date = new Date(parseInt(node.getAttribute('date'), 10) * 1000);
        var pubDate = new Date(parseInt(node.getAttribute('pubDate'), 10) * 1000);
        var commentCount = parseInt(node.getAttribute('commentCount'), 10);
        switch (type) {
        case 0:
            return new spyl.pb.Media.Picture(id, userId, date, pubDate, description, filename, commentCount);
        case 1:
            return new spyl.pb.Media.Movie(id, userId, date, pubDate, description, filename, commentCount);
        }
        throw new jls.lang.Exception('Invalid type (' + type + ')');
    }
});

spyl.pb.Media.Picture = jls.lang.Class.create(spyl.pb.Media,
{
    createElement : function(params, parent) {
        return new spyl.pb.PictureElement(params, parent);
    }
});

spyl.pb.Media.Movie = jls.lang.Class.create(spyl.pb.Media,
{
    createElement : function(params, parent) {
        return new spyl.pb.MovieElement(params, parent);
    }
});

